<?php

namespace FiloBlu\Flow\Block\Adminhtml\Channel\Edit\Tab;

class Connector extends \Magento\Backend\Block\Widget\Form\Generic implements \Magento\Backend\Block\Widget\Tab\TabInterface
{
    /**
     * @var \Magento\Store\Model\System\Store
     */
    protected $_systemStore;

    /**
     * Connector constructor.
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Data\FormFactory $formFactory
     * @param \Magento\Store\Model\System\Store $systemStore
     * @param \FiloBlu\Flow\Helper\Data $helper
     * @param array $data
     */

    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Data\FormFactory $formFactory,
        \Magento\Store\Model\System\Store $systemStore,
        \FiloBlu\Flow\Helper\Data $helper,
        array $data = []
    ) {
        $this->_systemStore = $systemStore;
        $this->helper = $helper;
        parent::__construct($context, $registry, $formFactory, $data);
    }

    /**
     * Prepare label for tab
     *
     * @return string
     */
    public function getTabLabel()
    {
        return __('Connector');
    }

    /**
     * Prepare title for tab
     *
     * @return string
     */
    public function getTabTitle()
    {
        return __('connector');
    }

    /**
     * {@inheritdoc}
     */
    public function canShowTab()
    {
        return true;
    }

    /**
     * {@inheritdoc}
     */
    public function isHidden()
    {
        return false;
    }

    /**
     * Prepare form
     *
     * @return $this
     */
    protected function _prepareForm()
    {
        /* @var $model \Magento\Cms\Model\Page */
        $model = $this->_coreRegistry->registry('flow_channel');

        /** @var \Magento\Framework\Data\Form $form */
        $form = $this->_formFactory->create();

        $form->setHtmlIdPrefix('page_');

        $fieldset_connector = $form->addFieldset('connector_fieldset', array('legend' => __('Connector')));


        $fieldset_connector->addField("type", "text", array(
            "label" => __("Type"),
            "name" => "connector[type]",
        ));

        $fieldset_connector->addField("host", "text", array(
            "label" => __("Host"),
            "name" => "connector[host]",
        ));

        $fieldset_connector->addField("username", "text", array(
            "label" => __("Username"),
            "name" => "connector[username]",
        ));

        $fieldset_connector->addField("password", "text", array(
            "label" => __("Password"),
            "name" => "connector[password]",
        ));

        $fieldset_connector->addField("passive", "select", array(
            "label" => __("Passive Mode"),
            "name" => "connector[passive]",
            'values' => array('0' => 'No', '1' => 'Yes'),
        ));

        $fieldset_connector->addField("ftp_dir", "text", array(
            "label" => __("Ftp Dir"),
            "name" => "connector[ftp_dir]",
        ));

        $fieldset_connector->addField("local_dir", "text", array(
            "label" => __("Local Dir"),
            "name" => "connector[local_dir]",
        ));

        $fieldset_connector->addField("regex", "text", array(
            "label" => __("File Mask"),
            "name" => "connector[regex]",
        ));


        $form->setValues($model->getData('connector'));
        $this->setForm($form);

        return parent::_prepareForm();
    }

    /**
     * Check permission for passed action
     *
     * @param string $resourceId
     * @return bool
     */
    protected function _isAllowedAction($resourceId)
    {
        return $this->_authorization->isAllowed($resourceId);
    }

//    public function getTabUrl()
//    {
//        return $this->getUrl('*/*/edit/', ['_current' => true]);
//    }

}
