<?php

namespace FiloBlu\Flow\Model\From\Entity;

abstract class AbstractEntity
{

    protected $specialAttributeRex = '^_';
    protected $_entityTypeCode = null;
    protected $_entityTypeModel = null;
    protected $_entityIdentifier = null;
    protected $_attributeCode = null;
    protected $_attributeValue = null;
    protected $_storeId = null;
    protected $_attributeModel = null;
    protected $_objectEntity = null;
    protected $_objectEntityId = null;
    protected $_product_factory;
    protected $_registry;
    protected $helper_attribute;

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \FiloBlu\Flow\Helper\Product $helper_product,
        \FiloBlu\Flow\Helper\Attribute $helper_attribute,
        array $data = []
    ) {
        $this->helper_product = $helper_product;
        $this->_storeId = $data['storeId'];
        $this->_attributeCode = trim($data['attributeCode']); // a
        $this->_attributeValue = trim($data['attributeValue']); // v
        $this->_entityIdentifier = $data['entityIdentifier']; // e
        $this->_entityTypeCode = $data['entityTypeCode']; // valori: catalog_product, product_attribute
        $this->_product_factory = $productFactory;
        $this->_registry = $registry;
        $this->helper_attribute = $helper_attribute;
        $this->getEntityModel();
    }

    protected function getEntityModel()
    {
        if (is_null($this->_entityTypeModel)) {
            $entityType = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Eav\Model\Entity\Type')->loadByCode($this->getEntityTypeCode());
            if (!$entityType->getId()) {
                throw new \Exception('EntityType not found for code ' . $this->_entityTypeCode);
            }

            $this->_entityTypeModel = $entityType->getEntityModel();
        }

        return $this->_entityTypeModel;
    }

    public function getEntityTypeCode()
    {
        if (empty($this->_entityTypeCode)) {
            throw new \Exception('EntityTypeCode was not defined for this class');
        }

        return $this->_entityTypeCode;
    }

    public function process()
    {

        if ($this->isSpecialFunction($this->_attributeCode)) {
            $result = $this->processSpecialFunction();
            if (is_string($result)) {
                throw new \Exception($result);
            }
        } else {
            $this->setObjectAttributeValue();
        }
    }

    /**
     * @param $attribute
     * @return false|int
     */
    protected function isSpecialFunction($attribute)
    {
        return preg_match('/' . $this->specialAttributeRex . '/', $attribute);
    }

    /**
     * @return mixed
     * @throws \Exception
     */
    protected function processSpecialFunction()
    {
        $className = $this->getSpecialActionClassName();

        try{
            $specificSpecialClass = \Magento\Framework\App\ObjectManager::getInstance()->create($className);
        }catch(\Exception $e){
            throw new \Exception("A special class for attribute code (" . $this->_attributeCode . ") was not found");
        }

        if (!method_exists($specificSpecialClass, 'process')) {
            throw new \Exception('Missing special function for attribute ' . $this->_attributeCode);
        }

        return $specificSpecialClass->process($this->getObject(), $this->_attributeValue, $this->_storeId);
    }

    /**
     * @return string
     */
    protected function getSpecialActionClassName()
    {
        $className = "\FiloBlu\Flow\Model\Actions\From" . "\\" . ucfirst(str_replace("_", "", $this->_attributeCode));

        return $className;
    }

    protected function getObject()
    {
        if (is_null($this->_objectEntity)) {
            if (!$this->getObjectId()) {
                throw new \Exception('No id found for model ' . $this->getEntityModel() . ' with given identifier ' . $this->_entityIdentifier);
            }
            $_prod_sku = $this->_product_factory->create()->getIdBySku($this->_entityIdentifier);
            $this->_objectEntity = $this->_product_factory->create()->setStoreId($this->_storeId)->load($_prod_sku);
            if (!$this->_objectEntity->getId()) {
                throw new \Exception('No instance found for model' . $this->getEntityModel() . ' with id ' . $this->getObjectId());
            }
        }
        return $this->_objectEntity;
    }

    public function getObjectId()
    {
        if (is_null($this->_objectEntityId)) {
            $this->_objectEntityId = $this->_entityIdentifier;
        }
        return $this->_objectEntityId;
    }

    protected function setObjectAttributeValue()
    {

        $object = $this->getObject();
        $attributeValue = $this->getAttributeValue();
        $input_type = strtolower($this->getAttribute()->getFrontend()->getInputType());
        try {
            if ($input_type === 'multiselect' && empty($attributeValue)) {
                $object->unsetData($this->_attributeCode);
                $object->save();
            } elseif (is_null($attributeValue) || empty($attributeValue)) {
                $object->unsetData($this->_attributeCode);
            } else {
                $object->setDataUsingMethod($this->_attributeCode, $attributeValue);
                $object->getResource()->saveAttribute($object, $this->_attributeCode);
            }
        } catch (\Exception $e) {
            $message = "Problem while processing eavs, <br/> store id : " . $this->_storeId . "<br/> id : "
                . $object->getId() . "<br/> attribute data : " . $attributeValue;
            $message .= "<br/> \Exception : " . $e->getMessage();
            $message .= "<br/> Backtrace : <br/> " . nl2br($e->getTraceAsString());
            throw new \Exception($e->getMessage());
        }
    }

    /**
     * @return null|string
     * @throws \Exception
     */
    protected function getAttributeValue()
    {

        $attribute_model = $this->getAttribute();
        $attributeValue = $this->_attributeValue;
        $input_type = strtolower($attribute_model->getFrontend()->getInputType());

        if (strlen($attributeValue) > 0 && ($input_type === 'select' || $input_type === 'multiselect') && $this->getAttributeCode() != 'visibility') {
            $attributeValueLabels = array();
            $attributeValueArray = array();
            if ($input_type === 'multiselect') {
                $attributeValueLabels = explode(',', $attributeValue);
                $attributeValueLabels = array_map('trim', $attributeValueLabels);
                $attributeValueLabels = array_unique($attributeValueLabels);
            } else {
                $attributeValueLabels[] = $attributeValue;
            }
            foreach ($attributeValueLabels as $attributeValueLabel) {
                $optionId = $this->getAttributeOptionValue($attributeValueLabel);
                if (empty($optionId) && $this->_storeId == 0) { // && Mage::getStoreConfig('fbflow/new_options_status') == 1) {
                    $this->addAttributeOptionValue($attributeValueLabel);
                    $optionId = $this->getAttributeOptionValue($attributeValueLabel);
                }
                if (empty($optionId)) {
                    throw new \Exception('Couldn\'t find nor add option ' . $attributeValueLabel . ' for attribute ' . $this->_attributeCode . ' entity type ' . $this->getEntityTypeCode() . ' on store ' . $this->_storeId);
                }
                $attributeValueArray[] = $optionId;
            }
            $attributeValue = implode(',', $attributeValueArray);
        }

        return $attributeValue;
    }

    /**
     * @return null
     * @throws \Exception
     */
    protected function getAttribute()
    {
        if (empty($this->_attributeModel)) {
            $this->_attributeModel = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Eav\Model\Entity\Attribute')->loadByCode($this->getEntityTypeCode(),
                $this->getAttributeCode());
            if (!$this->_attributeModel->getId()) {
                throw new \Exception('No attribute found for code ' . $this->_attributeCode);
            }
        }
        return $this->_attributeModel;
    }

    public function getAttributeCode()
    {
        if (empty($this->_attributeCode)) {
            throw new \Exception('AttributeCode was not defined for this class');
        }

        return $this->_attributeCode;
    }

    /**
     * @param $attributeValueLabel
     * @return mixed
     */
    protected function getAttributeOptionValue($attributeValueLabel)
    {

        $collection = \Magento\Framework\App\ObjectManager::getInstance()
            ->create('\Magento\Eav\Model\ResourceModel\Entity\Attribute\Option\Collection');

        $collection->getSelect()
            ->join('eav_attribute', 'eav_attribute.attribute_id=main_table.attribute_id',
                'eav_attribute.attribute_code')
            ->join('eav_attribute_option_value', 'eav_attribute_option_value.option_id=main_table.option_id', 'value');

        $collection
            ->addFieldToFilter('eav_attribute_option_value.value', array('eq' => $attributeValueLabel))
            ->addFieldToFilter('attribute_code', array('eq' => $this->_attributeCode))
            ->addFieldToFilter('store_id', array('eq' => $this->_storeId));

        $attribute = $collection->getFirstItem();

        return $attribute->getOptionId();
    }

    /**
     * @param $attributeValueLabel
     * @return bool
     * @throws \Exception
     */
    protected function addAttributeOptionValue($attributeValueLabel)
    {

        try {

            $data = array(
                'attribute_id' => $this->getAttribute()->getId(),
                'store_id' => 0,
                'value' => $attributeValueLabel
            );
            $attributeOptionId = $this->helper_attribute->addAttributeOption($data);

            return $attributeOptionId;

        } catch (\Exception $e) {
            throw new \Exception('Couldn\'t add option ' . $attributeValueLabel . ' for attribute ' . $this->_attributeCode . ' entity type ' . $this->getEntityTypeCode());
        }

    }

}

//End of class